    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % script file - multiple matrix multiplication model evolutionary
    % simulation.
    % This file sets all parameter values and calls the evolutionary
    % simulation file 'ga_multiple_layers.m'.
    %
    % Written by Tamar Friedlander
    % June 2014
    %
    % 
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% fast matrix multiplication package
addpath('C:\_my_files\Tamar_IST_Weizmann\Matlab\mtimesx_20110223') % path where 'mtimesx' is kept.
FLAGS.MEX = 1; % '1': 'mtimesx' matrix multiplication package is used. '0': use regular matrix multiplication instead of 'mtimesx'

GA.general.epsilon = 0; % weight parameter if cost on connections between network components is included.
GA.general.PopSize = 100; % size of population
GA.general.MaxGen = 10000; % maximal number of generations
GA.general.MatNum = 4; % Number of node layers in the network -1. Should be an integer number. The number of layers of nodes is + 1.
GA.general.rev_ord = 1;
GA.general.metric_FLAG = 'euclid'; % changed 21/3/2012: 'euclid' - sum square deviations (as before); 'max' - take the maximal deviation.
GA.general.StopCon = 'FitnessOnly'; %'FitnessOnly' - Stop by fitness criterion alone. 'None' - Stop only by number of generations.
GA.general.StopThreshold = 0.001;

% goal options
GA.goal.MatDim = 6; % matrix dimension
GA.goal.LB = 0; % lower bound to A values
GA.goal.UB = 0.1;  % upper bound to A values
GA.goal.GoalType = 1; % '0' - input-output pairs V and U are given. '1' - the desired system G is given instead. '2' - Temporally noisy goal. The noise realization changes every generation. 
GA.goal.TempNoise = 0; % added 31/3/2014: This option is in use if GA.goal.GoalType=2 (temporally changing goal0. In this case the exact goal is calculated in the function 'ga_multiple_layers'.
G = ones(GA.goal.MatDim) * 10 ; % goal matrix definition. r = 1


% initial population
GA.init.FileFLAG = 0; % '0' - start from random population, '1' - load initial population from file.
% mutation options: 
%mutation could be either additive or multiplicative. 
GA.mutation.MutType = 2; % '2' additive mutations drawn from a Gaussian around 0, 
% '2' - multiplicative mutations drawn from a Gaussian around 1 + a small additive mutation, 
% '3' - multiplicative mutations drawn from a log-normal distribution. 
% '4' - additive non-negative - absorbing boundary.
% '5' - additive non-negative mutations with reflecting boundary, i.e. taking the absolute value of the mutation
% value, rather than the maximum between 0 and the mutation value.
GA.mutation.MutRate_A = 0.05/GA.goal.MatDim.^2;  % probability of mutations in A elements.
GA.mutation.MeanMut = 1; % mean mutation value. Applicable only to multipilicative mutations.
GA.mutation.MutSize_A = 0.1; % mutations are randomly chosen from a Gaussian distribution N(0,MutSize);
% These 2 parameters are currently unsed
GA.mutation.MutSize2_A = 0; %0.01; % used only if GA.mutation.MutType = 2 or 3, to represent the size of the additional additive mutation in A elements.
GA.mutation.sgn_switch_prob = 0;

% selection
GA.selection.method = 'Tournament'; % options: 'Roulette' - roulette wheel selection; 'Tournament' - chooses with uniform distribution sets of 's' individuals each and takes the fittest from each set.
GA.selection.tournament_s = 4; % Parameter used only if GA.selection.method = 'Tournament'. 's' determines the selection strength.
GA.selection.fitscale = 'None'; % options: 'None' - scaled_fval = f; 'Exponential' take exp(beta*f)/sum(exp(beta*f)).
GA.selection.beta = 1; % "1/temperature" taken in scaling the population fitness. Pay attention: If beta values are too high (~500) the scaled fitness might be Nan.
% This parameter is used only if GA.selection.fitscale = 'Exponential'.
% zero counting 

% simulation output saving parameters
FLAGS.save_FLAG = 1; % save data or not
FLAGS.save.full = 1; % save temporal fitness value. Otherwise saves only final matrices. 
GA.general.save_path = 'C:\_my_files\Tamar_IST_Weizmann\my_texts\bow-ties\MSB\simulation_code';  % directory where the data will be stored.

% noise
GA.Noise.TempNoiseFLAG = 0; 
GA.Noise.Intensity = 0;
GA.Noise.Intensity = 0;
GA.goal.TempNoise = 0;


for kk = 1:1 % stochastic repeats of the simulation. The Matlab random seed is set to a different value of 'kk' each time.
    RandSeed = kk
    %when changing matrix dimension, all other parameters that depend on
    %them should be recalculated.
    GA.mutation.MutRate_A = 0.05 / GA.goal.MatDim.^2;  % probability of mutations in A elements scales with matrix size to keep constant element-wise mutation rate.
    % define where the simulation code is kept
    cd 'C:\_my_files\Tamar_IST_Weizmann\my_texts\bow-ties\MSB\simulation_code\code_clean_ver'
    fname = ['datafile_',num2str(RandSeed)]; 
    ga_multiple_layers(kk, GA, FLAGS, G, fname);
end;   % for kk = 1:10,

%send an email at the end of the run
setpref('Internet','SMTP_Server','ownew.ist.ac.at');
setpref('Internet','E_mail','tamar.friedlander@ist.ac.at');
sendmail('tamar.friedlander@ist.ac.at',['End of run, Number of repeats = ',num2str(kk)]);

   
