function [F, del_F_over_F] = node_effect_on_fitness(A, G, rev_ord)

% This function gets as an input the results of evolutionary simulation in
% multiple layer model. It should decide which nodes could be eliminated with little effect on fitness.
% To do it, we eliminate a sinle node each time by equating t ozero all its
% input and output interactions and calculate the effect on fitness. 

%  A is a cell with dimensions (1,L). A{i} is a square matrix with
%  dimension D. 
% The # of matrices in the model is L.
%  the # of node layers is L+1.
% the # of nodes in each layer is the dimension of the matrices.

% 16/7/2013: the parameter 'rev_ord' was added. I reversed the order of matrix multiplication to be consistent
% with other works. Until now it was A{1}*A{2}*A{3}*...A{L} so that the input
% was layer L and output was layer 1.
% From now on it is A{L}*A{L-1}*....*A{1} so that input is layer a and
% output is layer L. If the flag rev_ord =0 assume the old order. Other
% wise use the new order.

% My convention to enumarete the nodes: there are L+1 layers of nodes.
% n{1} is input which interacts with A{1} (its output)
% n{L+1} is output which interacts only with A{L} (its input)
% intermediate layers n{l} interact with both A{l-1} (its input) and A{l}
% (its output).

% 23/7/2013: The new parameter 'Method' enables to choose between different
% ways of determining which interaction are inactive. Possible values:
% '1' - calculate all interactions in which a term is involved and compare
% the maximal interaction to a given threshold.
% '2' - eliminate an interaction and then determine the effect on fitness
% with and without this term. 

% 12/12/13: Corrections made to handle non-square matrices or different
% matrix dimensions at different layers.
% Warning: so far this was oonly teted for rev_ord = 1, but NOT for rev_ord
% = 0.

% 30/4/2014: temporal noise intensity arguments added to the calls to 'fitness_mult' due to change in 'fitness_mult'.

% Written by Tamar Friedlander
% 28/7/2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% if ~isempty(varargin)
%     varargin{1} = Method;
%     varargin{2} = G;
% else
%     Method = 1;
%     %G =[];
% end;

MEX = 1;
L = length(A);
D = size(A{1},1);
%A_z = cell(1,L);
%nodes = ones(L+1,D); % nodes in the network. Each row is network layer.
F0 = fitness_mult(A, G, rev_ord, MEX, 0, 0);  % 30/4/2014: temporal noise intensity arguments added due to change in 'fitness_mult'.
F = zeros(L+1,D); % modified fitness if a single node is eliminated.
D_output = size(A{L},1);
%if Method == 1 % determine maximal interaction intensity for each term.
    for ll = 1:L+1
        %A_z{ll} = zeros(D);
        if ll < L + 1 % node number can vary between layers
            node_num = size(A{ll},2);
        elseif ll == L+1,
            node_num = D_output;
        end; % if ll < L + 1
        for kk = 1 : node_num, % 12/12/13: corrected to handle non-square matrices
            M = A; 
            if rev_ord == 1, % then layer 1 is input and layer L+1 is output. 
                if ll > 1 % if not the input layer
                    M{ll-1}(kk,:) = 0 * M{ll-1}(kk,:); %zeros(1,D); % this row is the input to the node
                end;
                if ll < L + 1 % if not output layer
                    M{ll}(:,kk) = 0 * M{ll}(:,kk); %zeros(D,1); % this column is the output from the node
                end;
            elseif rev_ord == 0, % layer 1 is output and layer L+1 is input.
                if ll > 1 % not input
                    M{ll-1}(:,kk) = 0 * M{ll-1}(:,kk); %zeros(D,1); % output from the node
                end;
                if ll < L + 1,
                    M{ll}(kk,:) = 0 * M{ll}(kk,:); %zeros(1,D); % input to the node
                end;
            end;  % if rev_ord == 1,
            if size(M{1},1)~=size(A{1},1) || size(M{1},2)~=size(A{1},2)
                error('Unmatching size!!!');
            else
                F(ll,kk) = fitness_mult(M, G, rev_ord, MEX, 0, 0); % 30/4/2014: temporal noise intensity arguments added due to change in 'fitness_mult'.
            end;
        end;  % for kk=1:D
    end; % for ll=1:L+1
        
    del_F_over_F = abs((F0 - F)/F0);
% elseif Method == 2, % determine the effect on fitness
%     Threshold2 = 0.1 / L / D^2;
%     
%     for ll = 1:L
%         A_z{ll} = zeros(D);
%         for ii = 1 : D,
%             for jj = 1:D
%                 M = A; 
%                 M{ll}(ii,jj) = 0; % eliminate a single term and leave the others intact.
%                 F{ll}(ii,jj) = fitness_mult(M, G, rev_ord, MEX);
%             end;
%         end;
%     end; %  for ll = 1:L
%         del_F_over_F = cellfun(@(x) abs((F0-x)/F0), F, 'UniformOutput',false);
%         %A_z = cellfun(@(x) x(x > Threshold2), del_F_over_F, 'UniformOutput',false);
%         for ll = 1: L
%             Az_temp = A{ll};
%             Az_temp(del_F_over_F{ll} < threshold) = 0;
%             A_z{ll} = Az_temp;
%         end;

end

